@@ -11,6 +11,10 @@
 #elif FAT
 #elif NTFS
     #include <ntfs/version.h>
+#elif VMFS
+    #include <vmfs/vmfs.h>
+#elif BTRFS
+    #include "btrfs/version.h"
 #endif
 
 int main(int argc, char **argv){
@@ -30,8 +34,12 @@ int main(int argc, char **argv){
 	err = libextfs_version();
     } else if (strcmp(libfs, "reiserfs") == 0) {
 	err = libreiserfs_version();
+    } else if (strcmp(libfs, "vmfs") == 0) {
+	err = libvmfs_version();
     } else if (strcmp(libfs, "reiser4") == 0) {
 	err = libpreiser4_version();
+    } else if (strcmp(libfs, "btrfs") == 0) {
+	err = libbtrfs_version();
     }
 
     if(err == 1){
@@ -41,6 +49,17 @@ int main(int argc, char **argv){
 
 }
 
+int libvmfs_version(){
+#ifdef VMFS
+    char *version;
+    version = "0.2.0";
+#ifdef VMFS5_ZLA_BASE
+    version = "0.2.5";
+#endif
+    printf("%s\n", version);
+#endif
+}
+
 int libntfs_version(){
 #ifdef NTFS
     printf("%s\n", ntfs_libntfs_version());
@@ -79,3 +98,11 @@ int libreiserfs_version(){
 #endif
     return 1;
 }
+
+int libbtrfs_version(){
+#ifdef BTRFS
+    printf("%s\n", BTRFS_BUILD_VERSION);
+    return 0;
+#endif
+    return 1;
+}
