@@ -26,6 +26,7 @@
 #include "progress.h"
 #include "fs_common.h"
 
+#undef crc32
 char	*EXECNAME = "partclone.xfs";
 extern  fs_cmd_opt fs_opt;
 int	source_fd = -1;
@@ -39,6 +40,36 @@ unsigned int    source_sectorsize;      /* source disk sectorsize */
 
 #define rounddown(x, y) (((x)/(y))*(y))
 
+void get_sb(xfs_sb_t *sbp, xfs_off_t off, int size, xfs_agnumber_t agno)
+{
+        xfs_dsb_t *buf;
+	int rval = 0;
+
+        buf = memalign(libxfs_device_alignment(), size);
+        if (buf == NULL) {
+                log_mesg(3, 1, 1, fs_opt.debug, "error reading superblock %u -- failed to memalign buffer\n", agno);
+        }
+        memset(buf, 0, size);
+        memset(sbp, 0, sizeof(*sbp));
+
+        /* try and read it first */
+
+        if (lseek64(source_fd, off, SEEK_SET) != off)  {
+                log_mesg(3, 0, 1, fs_opt.debug, "error reading superblock %u -- seek to offset %" PRId64 " failed\n", agno, off);
+                free(buf);
+        }
+
+        if ((rval = read(source_fd, buf, size)) != size)  {
+                log_mesg(3, 1, 1, fs_opt.debug, "superblock read failed, offset %" PRId64 ", size %d, ag %u, rval %d\n", off, size, agno, rval);
+        }
+        libxfs_sb_from_disk(sbp, buf);
+
+        //rval = verify_sb((char *)buf, sbp, agno == 0);
+        free(buf);
+        //return rval;
+}
+
+
 static void set_bitmap(unsigned long* bitmap, uint64_t pos, int length)
 {
     uint64_t pos_block;
@@ -61,7 +92,7 @@ static void fs_open(char* device)
     int		    open_flags;
     struct stat     statbuf;
     int		    source_is_file = 0;
-    xfs_buf_t       *sbp;
+    //xfs_buf_t       *sbp;
     xfs_sb_t        *sb;
     int             tmp_residue;
 
@@ -101,10 +132,14 @@ static void fs_open(char* device)
     }
 
     /* prepare the mount structure */
-    sbp = libxfs_readbuf(xargs.ddev, XFS_SB_DADDR, 1, 0);
     memset(&mbuf, 0, sizeof(xfs_mount_t));
     sb = &mbuf.m_sb;
-    libxfs_sb_from_disk(sb, XFS_BUF_TO_SBP(sbp));
+    get_sb(sb, 0, XFS_MAX_SECTORSIZE, 0);
+    //sbp = libxfs_readbuf(xargs.ddev, XFS_SB_DADDR, 1, 0, ops);
+
+    //memset(&mbuf, 0, sizeof(xfs_mount_t));
+    //sb = &mbuf.m_sb;
+    //libxfs_sb_from_disk(sb, XFS_BUF_TO_SBP(sbp));
 
     mp = libxfs_mount(&mbuf, sb, xargs.ddev, xargs.logdev, xargs.rtdev, 1);
     if (mp == NULL) {
@@ -177,13 +212,12 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
     int             read_ag_length;
     void            *read_ag_buf = NULL;
     xfs_off_t	    read_ag_position;            /* xfs_types.h: typedef __s64 */
-    uint64_t	    sk, res, s_pos = 0;
+    uint64_t	    sk, res;
     void            *btree_buf_data = NULL;
     int		    btree_buf_length;
     xfs_off_t	    btree_buf_position;
     xfs_agblock_t   bno;
     uint	    current_level;
-    uint	    btree_levels;
     xfs_daddr_t     begin, next_begin, ag_begin, new_begin, ag_end;
 						/* xfs_types.h: typedef __s64*/
     xfs_off_t       pos;
@@ -232,6 +266,9 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 	read_ag_length = first_agbno * source_blocksize;
 	read_ag_position = (xfs_off_t) read_ag_off * (xfs_off_t) BBSIZE;
 	read_ag_buf = malloc(read_ag_length);
+	if(read_ag_buf == NULL){
+	    log_mesg(0, 1, 1, fs_opt.debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+	}
 	memset(read_ag_buf, 0, read_ag_length);
 
 	log_mesg(2, 0, 0, fs_opt.debug, "seek to read_ag_position %lli\n", read_ag_position);
@@ -257,6 +294,9 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 	/* save what we need (agf) in the btree buffer */
 
 	btree_buf_data = malloc(source_blocksize);
+	if(btree_buf_data == NULL){
+	    log_mesg(0, 1, 1, fs_opt.debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
+	}
 	memset(btree_buf_data, 0, source_blocksize);
 	memmove(btree_buf_data, ag_hdr.xfs_agf, source_sectorsize);
 	ag_hdr.xfs_agf = (xfs_agf_t *) btree_buf_data;
@@ -266,7 +306,6 @@ extern void readbitmap(char* device, image_head image_hdr, unsigned long* bitmap
 
 	bno = be32_to_cpu(ag_hdr.xfs_agf->agf_roots[XFS_BTNUM_BNOi]);
 	current_level = 0;
-	btree_levels = be32_to_cpu(ag_hdr.xfs_agf->agf_levels[XFS_BTNUM_BNOi]);
 
 	ag_end = XFS_AGB_TO_DADDR(mp, agno, be32_to_cpu(ag_hdr.xfs_agf->agf_length) - 1) + source_blocksize / BBSIZE;
 
